#include <disphelper.c>

typedef struct stringInfo
{
char rtnStg[256];
} stringData;

stringData sd;
stringData * sdPtr;

typedef struct textInfo
{
char rtnText[6000];
} textData;

textData td;
textData * tdPtr;

typedef struct wordInfo
{
char rtnWord[512];
} wordData;

wordData wd;
wordData * wdPtr;

struct OCRInfo
{
int languageid;
int words;
int characters;
int fonts;
int images;
int bitsperpixel;
int pixelwidth;
int pixelheight;
int xdpi;
int ydpi;
int compression;
};

struct PageInfo
{
int reserved;
int lineid;
int regionid;
int fontid;
int id;
};

struct RectInfo
{
int left;
int top;
int right;
int bottom;
};

extern BOOL COMAL_Debug(BOOL);
extern stringData * COMAL_GetErrorCode(void);
extern stringData * COMAL_GetErrorMsg(void);

extern BOOL OCR_Close(void);
extern BOOL OCR_PerformOCR(void);
extern textData * OCR_GetText(int);
extern wordData * OCR_GetWord(int,int);
extern void OCR_GetOCRInfo(int,struct OCRInfo *ocrPtr);
extern void OCR_GetPageInfo(int,int,struct PageInfo *pagePtr);
extern void OCR_GetRectangleInfo(int,int,int,struct RectInfo *rectPtr);
extern BOOL OCR_SaveAsMDI(char *);
extern BOOL OCR_Open(char *);

extern stringData * COMALOCR_GetVersion(BYTE);

DISPATCH_OBJ(modiDoc);

int rtn=0,record=0;
BOOL ocrSts=FALSE;

char * UnicodeToAnsi(OLECHAR * szW)
{
static char szA[256];
szA[0]=0;
if (szW==NULL)
return "";
WideCharToMultiByte(CP_ACP,0,szW,-1,szA,256,NULL,NULL);
return szA;
}

BOOL __declspec(dllexport) COMAL_Debug(BOOL onoff)
{
HRESULT result;
result=dhToggleExceptions(onoff);
return result;
}

stringData * __declspec(dllexport) COMAL_GetErrorCode(void)
{
PDH_EXCEPTION pException=NULL;
dhGetLastException(&pException);
if (pException)
sprintf(sd.rtnStg,"%lx",pException->swCode);
else
sprintf(sd.rtnStg,"NO ERROR");
return &sd;
}

stringData * __declspec(dllexport) COMAL_GetErrorMsg(void)
{
PDH_EXCEPTION pException=NULL;
dhGetLastException(&pException);
if (pException)
sprintf(sd.rtnStg,"%ls",pException->szDescription);
else
sprintf(sd.rtnStg,"NO ERROR");
return &sd;
}

BOOL __declspec(dllexport) OCR_Close(void)
{
if (ocrSts==TRUE)
{
dhCallMethod(modiDoc,L".Close");
SAFE_RELEASE(modiDoc);
ocrSts=FALSE;
dhUninitialize(TRUE);
return TRUE;
}
return FALSE;
}

BOOL __declspec(dllexport) OCR_PerformOCR(void)
{
if (ocrSts==TRUE)
{
if (SUCCEEDED(dhCallMethod(modiDoc,L".Images(%d).OCR()",0)))
return TRUE;
}
return FALSE;
}

textData * __declspec(dllexport) OCR_GetText(int pageNumber)
{
if (ocrSts==TRUE)
{
HRESULT result;
LPWSTR name=NULL;
char *namePtr=NULL;
result=dhGetValue(L"%S",&name,modiDoc,L".Images(%d).Layout.Text",pageNumber);
namePtr=UnicodeToAnsi(name);
strcpy(td.rtnText,namePtr);
dhFreeString(name);
if (FAILED(result))
return NULL;
if (strcmp(td.rtnText,"")==0)
return NULL;
return &td;
}
return NULL;
}

wordData * __declspec(dllexport) OCR_GetWord(int pageNumber,int wordNumber)
{
if (ocrSts==TRUE)
{
HRESULT result;
LPWSTR name=NULL;
char *namePtr=NULL;
result=dhGetValue(L"%S",&name,modiDoc,L".Images(%d).Layout.Words(%d).Text",pageNumber,wordNumber);
namePtr=UnicodeToAnsi(name);
strcpy(wd.rtnWord,namePtr);
dhFreeString(name);
if (FAILED(result))
return NULL;
if (strcmp(wd.rtnWord,"")==0)
return NULL;
return &wd;
}
return NULL;
}

void __declspec(dllexport) OCR_GetOCRInfo(int pageNumber,struct OCRInfo *ocrPtr)
{
if (ocrSts==TRUE)
{
int number=0;
rtn=dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.Language",pageNumber);
ocrPtr->languageid=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.NumWords",pageNumber);
ocrPtr->words=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.NumChars",pageNumber);
ocrPtr->characters=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.NumFonts",pageNumber);
ocrPtr->fonts=number;
dhGetValue(L"%d",&number,modiDoc,L".Images.Count");
ocrPtr->images=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).BitsPerPixel",pageNumber);
ocrPtr->bitsperpixel=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).PixelWidth",pageNumber);
ocrPtr->pixelwidth=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).PixelHeight",pageNumber);
ocrPtr->pixelheight=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).XDPI",pageNumber);
ocrPtr->xdpi=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).YDPI",pageNumber);
ocrPtr->ydpi=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Compression",pageNumber);
ocrPtr->compression=number;
}
}

void __declspec(dllexport) OCR_GetPageInfo(int pageNumber,int wordNumber,struct PageInfo *pagePtr)
{
if (ocrSts==TRUE)
{
int number=0;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.Words(%d).LineId",pageNumber,wordNumber);
pagePtr->lineid=number;
dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.Words(%d).RegionId",pageNumber,wordNumber);
pagePtr->regionid=number;
rtn=dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.Words(%d).FontId",pageNumber,wordNumber);
pagePtr->fontid=number;
rtn=dhGetValue(L"%d",&number,modiDoc,L".Images(%d).Layout.Words(%d).Id",pageNumber,wordNumber);
pagePtr->id=number;
}
}

void __declspec(dllexport) OCR_GetRectangleInfo(int pageNumber,int wordNumber,int rectNumber,struct RectInfo *rectPtr)
{
if (ocrSts==TRUE)
{
int number=0;
DISPATCH_OBJ(modiRect);
dhGetValue(L"%o",&modiRect,modiDoc,L".Images(%d).Layout.Words(%d).Rects(%d)",pageNumber,wordNumber,rectNumber);
dhGetValue(L"%d",&number,modiRect,L".Left");
rectPtr->left=number;
dhGetValue(L"%d",&number,modiRect,L".Top");
rectPtr->top=number;
dhGetValue(L"%d",&number,modiRect,L".Right");
rectPtr->right=number;
dhGetValue(L"%d",&number,modiRect,L".Bottom");
rectPtr->bottom=number;
SAFE_RELEASE(modiRect);
}
}

BOOL __declspec(dllexport) OCR_SaveAsMDI(char *fname)
{
if (ocrSts==TRUE)
{
if (SUCCEEDED(dhCallMethod(modiDoc,L".SaveAs(%s,%d,%d)",fname,4,1)))
{
dhCallMethod(modiDoc,L".Save");
return TRUE;
}
}
return FALSE;
}

BOOL __declspec(dllexport) OCR_Open(char *fname)
{
dhInitialize(TRUE);
if (ocrSts==FALSE)
{
if (SUCCEEDED(dhCreateObject(L"MODI.Document",NULL,&modiDoc)))
{
if (SUCCEEDED(dhCallMethod(modiDoc,L".Create(%s)",fname)))
{
ocrSts=TRUE;
return TRUE;
}
}
}
return FALSE;
}

stringData * __declspec(dllexport) COMALOCR_GetVersion(BYTE verstg)
{
if (verstg==1)
strcpy(sd.rtnStg,"http://www.yoingco.com/");
else if (verstg==2)
strcpy(sd.rtnStg,"john@yoingco.com");
else
strcpy(sd.rtnStg,"ComalOCR.dll V1.03 - (c) John White, 2006. All Rights Reserved.");
return &sd;
}

BOOL WINAPI __declspec(dllexport) LibMain(HINSTANCE hDLLInst,DWORD fdwReason,LPVOID lpvReserved)
{
switch (fdwReason)
{
case DLL_PROCESS_ATTACH:

break;
case DLL_PROCESS_DETACH:

break;
case DLL_THREAD_ATTACH:

break;
case DLL_THREAD_DETACH:

break;
}
return TRUE;
}
