#include <commctrl.h>
#include <string.h>
#include <wchar.h>
#include <objbase.h>
#include <initguid.h>
#include <tchar.h>
#include <mmsystem.h>

#define GENDER_NEUTRAL 0
#define GENDER_FEMALE 1
#define GENDER_MALE 2
#define TTSAGE_BABY 1
#define TTSAGE_TODDLER 3
#define TTSAGE_CHILD 6
#define TTSAGE_ADOLESCENT 14
#define TTSAGE_ADULT 30
#define TTSAGE_ELDERLY 70
#define TTS_EVENT_TEXTSTART 1
#define TTS_EVENT_TEXTDONE 2
#define ARRAY_SIZE 200
#define PLATFORM_WIN32 0
#define PLATFORM_WIN32_WINDOWS 1
#define PLATFORM_WIN32_NT 2
#define PLATFORM_WIN32_CE 3

typedef struct voiceInfo
{
char maker[256];
char product[256];
char modeName[256];
char speaker[256];
char style[256];
char dialect[64];
char gender[16];
char age[16];
DWORD features;
DWORD interfaces;
DWORD engineFeatures;
} voiceData;

typedef struct voiceMode
{
char modeName[256];
} voiceName;

voiceData vd[ARRAY_SIZE];
voiceData * vdPtr;
voiceName vn[ARRAY_SIZE];
voiceName * vnPtr;

extern int Begin_Speech(DWORD);
extern int Count_Voices(void);
extern void Enum_VoiceData(void);
extern void Enum_VoiceNames(void);
extern voiceData * Get_VoiceData(int);
extern voiceName * Get_VoiceName(int);
extern BOOL Speak_String(int,char *,int,int,int,int,DWORD,int,DWORD,int,DWORD);
extern BOOL Pause_Speech(void);
extern BOOL Stop_Speech(void);
extern BOOL Resume_Speech(void);
extern BOOL Tag_Speech(char *);
extern void Delete_Speech(void);
extern DWORD Get_Volume(int);
extern DWORD Get_Pitch(int);
extern DWORD Get_Speed(int);
extern void End_Speech(void);

typedef struct _TtsModeInfoW
{
GUID gEngineID;
WCHAR szMfgName[262];
WCHAR szProductName[262];
GUID gModeID;
WCHAR szModeName[262];
LANGID LanguageID;
WCHAR szDialect[64];
WCHAR szSpeaker[262];
WCHAR szStyle[262];
WORD wGender;
WORD wAge;
DWORD dwFeatures;
DWORD dwInterfaces;
DWORD dwEngineFeatures;
} TTSMODEINFOW, *PTTSMODEINFOW;

typedef void *TTSENUM;
typedef void *TTS;
typedef void (* TTSCALLBACK)(DWORD dwEvent);

TTSENUM tts_EnumCreate(void);
void tts_EnumDelete(TTSENUM ttsEnum);
BOOL tts_EnumReset(TTSENUM ttsEnum);
BOOL tts_EnumNext(TTSENUM ttsEnum,PTTSMODEINFOW pTtsModeInfo);
TTS tts_Create(PTTSMODEINFOW pTtsModeInfo,TTSCALLBACK lpfnCallback);
void tts_Delete(TTS Tts);
BOOL tts_Stop(TTS Tts);
BOOL tts_Pause(TTS Tts);
BOOL tts_Resume(TTS Tts);
BOOL tts_Inject(TTS Tts,WCHAR *);
BOOL tts_TextOut(TTS Tts);
BOOL tts_SetVolume(TTS Tts,DWORD nVolPercent);
BOOL tts_GetVolume(TTS Tts,DWORD *pVolPercent);
BOOL tts_SetSpeed(TTS Tts,DWORD nSpeedPercent);
BOOL tts_GetSpeed(TTS Tts,DWORD *pSpeedPercent);
BOOL tts_SetPitch(TTS Tts,DWORD nPitchPercent);
BOOL tts_GetPitch(TTS Tts,DWORD *pPitchPercent);

char * UnicodeToAnsi(OLECHAR *);
voiceData * Enum_VoiceInfo(void);
voiceName * Enum_VoiceMode(void);

static HINSTANCE hInstance;
static TTS tts=NULL;            // Static, so it retains values between calls to this function.
HGLOBAL Memory_Handle;
char * Memory_LockPtr;
HGLOBAL MemCopy_Handle;
char * MemCopy_LockPtr;
int initd=-1,rtn=0;
DWORD memSize=0,cdid=0,wdid=0,mdid=0;
BYTE dcg=0,mcg=0;

//------------------------------------------ Speech Declarations ------------------------------------------

#ifndef _MBCS
#ifndef _UNICODE
#define _MBCS
#endif
#endif

typedef unsigned __int64 QWORD, *PQWORD;

typedef struct SData
{
PVOID pData;
DWORD dwSize;
} SDATA, *PSDATA;

typedef enum
{
CHARSET_TEXT           = 0,
CHARSET_IPAPHONETIC    = 1,
CHARSET_ENGINEPHONETIC = 2
} VOICECHARSET;

#define CINTERFACE

DEFINE_GUID(CLSID_MMAudioDest, 0xcb96b400, 0xc743, 0x11cd, 0x80, 0xe5, 0x0, 0xaa, 0x0, 0x3e, 0x4b, 0x50);
DEFINE_GUID(IID_IAudioMultiMediaDevice, 0xb68ad320, 0xc743, 0x11cd, 0x80, 0xe5, 0x0, 0xaa, 0x0, 0x3e, 0x4b, 0x50);
DEFINE_GUID(CLSID_TTSEnumerator, 0xd67c0280, 0xc743, 0x11cd, 0x80, 0xe5, 0x0, 0xaa, 0x0, 0x3e, 0x4b, 0x50);
DEFINE_GUID(IID_ITTSEnumW, 0x6B837B20L, 0x4A47, 0x101B, 0x93, 0x1A, 0x00, 0xAA, 0x00, 0x47, 0xBA, 0x4F);
DEFINE_GUID(IID_ITTSCentralW, 0x28016060L, 0x4A47, 0x101B, 0x93, 0x1A, 0x00, 0xAA, 0x00, 0x47, 0xBA, 0x4F);
DEFINE_GUID(IID_ITTSNotifySinkW, 0xC0FA8F40L, 0x4A46, 0x101B, 0x93, 0x1A, 0x00, 0xAA, 0x00, 0x47, 0xBA, 0x4F);
DEFINE_GUID(IID_ITTSBufNotifySink, 0xe4963d40, 0xc743, 0x11cd, 0x80, 0xe5, 0x0, 0xaa, 0x0, 0x3e, 0x4b, 0x50);
DEFINE_GUID(IID_ITTSAttributesW, 0x1287A280L, 0x4A47, 0x101B, 0x93, 0x1A, 0x00, 0xAA, 0x00, 0x47, 0xBA, 0x4F);

#undef INTERFACE
#define INTERFACE IAudioMultiMediaDevice

DECLARE_INTERFACE_ (IAudioMultiMediaDevice,IUnknown)
{
// IUnknown members
STDMETHOD (QueryInterface) (THIS_ REFIID riid, LPVOID FAR* ppvObj) PURE;
STDMETHOD_(ULONG,AddRef)   (THIS) PURE;
STDMETHOD_(ULONG,Release)  (THIS) PURE;
// IAudioMultiMediaDevice members
STDMETHOD (CustomMessage)  (THIS_ UINT, SDATA) PURE;
STDMETHOD (DeviceNumGet)   (THIS_ DWORD*) PURE;
STDMETHOD (DeviceNumSet)   (THIS_ DWORD) PURE;
};
typedef IAudioMultiMediaDevice FAR * PIAUDIOMULTIMEDIADEVICE;

#undef INTERFACE
#define INTERFACE ITTSCentralW
DECLARE_INTERFACE_ (ITTSCentralW, IUnknown)
{
// IUnknown members
STDMETHOD (QueryInterface) (THIS_ REFIID riid, LPVOID FAR* ppvObj) PURE;
STDMETHOD_(ULONG,AddRef)   (THIS) PURE;
STDMETHOD_(ULONG,Release)  (THIS) PURE;
// ITTSCentralW members
STDMETHOD (Inject)         (THIS_ PCWSTR) PURE;
STDMETHOD (ModeGet)        (THIS_ PTTSMODEINFOW) PURE;
STDMETHOD (Phoneme)        (THIS_ VOICECHARSET, DWORD, SDATA, PSDATA) PURE;
STDMETHOD (PosnGet)        (THIS_ PQWORD) PURE;
STDMETHOD (TextData)       (THIS_ VOICECHARSET, DWORD, SDATA, PVOID, IID) PURE;
STDMETHOD (ToFileTime)     (THIS_ PQWORD, FILETIME *) PURE;
STDMETHOD (AudioPause)     (THIS) PURE;
STDMETHOD (AudioResume)    (THIS) PURE;
STDMETHOD (AudioReset)     (THIS) PURE;
STDMETHOD (Register)       (THIS_ PVOID, IID, DWORD*) PURE;
STDMETHOD (UnRegister)     (THIS_ DWORD) PURE;
};
typedef ITTSCentralW FAR * PITTSCENTRALW;

#undef INTERFACE
#define INTERFACE ITTSEnumW
DECLARE_INTERFACE_ (ITTSEnumW, IUnknown)
{
// IUnknown members
STDMETHOD (QueryInterface) (THIS_ REFIID riid, LPVOID FAR* ppvObj) PURE;
STDMETHOD_(ULONG,AddRef)   (THIS) PURE;
STDMETHOD_(ULONG,Release)  (THIS) PURE;
// ITTSEnumW members
STDMETHOD (Next)           (THIS_ ULONG, PTTSMODEINFOW, ULONG *) PURE;
STDMETHOD (Skip)           (THIS_ ULONG) PURE;
STDMETHOD (Reset)          (THIS) PURE;
STDMETHOD (Clone)          (THIS_ ITTSEnumW * FAR *) PURE;
STDMETHOD (Select)         (THIS_ GUID, PITTSCENTRALW *, LPUNKNOWN) PURE;
};

#undef INTERFACE
#define INTERFACE ITTSNotifySinkW
DECLARE_INTERFACE_ (ITTSNotifySinkW, IUnknown)
{
STDMETHOD (QueryInterface) (THIS_ REFIID riid, LPVOID FAR* ppvObj) PURE;
STDMETHOD_(ULONG,AddRef)   (THIS) PURE;
STDMETHOD_(ULONG,Release)  (THIS) PURE;
STDMETHOD (AttribChanged)  (THIS_ DWORD) PURE;
STDMETHOD (AudioStart)     (THIS_ QWORD) PURE;
STDMETHOD (AudioStop)      (THIS_ QWORD) PURE;
STDMETHOD (Visual)         (THIS_ QWORD, WCHAR, WCHAR, DWORD, LPVOID) PURE;
};
typedef ITTSNotifySinkW FAR * PITTSNOTIFYSINKW;

#undef INTERFACE
#define INTERFACE ITTSBufNotifySink
DECLARE_INTERFACE_ (ITTSBufNotifySink, IUnknown)
{
STDMETHOD(QueryInterface)  (THIS_ REFIID riid, LPVOID FAR* ppvObj) PURE;
STDMETHOD_(ULONG,AddRef)   (THIS) PURE;
STDMETHOD_(ULONG,Release)  (THIS) PURE;
STDMETHOD (TextDataDone)   (THIS_ QWORD, DWORD) PURE;
STDMETHOD (TextDataStarted)(THIS_ QWORD) PURE;
STDMETHOD (BookMark)       (THIS_ QWORD, DWORD) PURE;
STDMETHOD (WordPosition)   (THIS_ QWORD, DWORD) PURE;
};
typedef ITTSBufNotifySink FAR * PITTSBUFNOTIFYSINK;

#undef INTERFACE
#define INTERFACE ITTSAttributesW
DECLARE_INTERFACE_ (ITTSAttributesW, IUnknown)
{
STDMETHOD(QueryInterface)  (THIS_ REFIID riid, LPVOID FAR* ppvObj) PURE;
STDMETHOD_(ULONG,AddRef)   (THIS) PURE;
STDMETHOD_(ULONG,Release)  (THIS) PURE;
STDMETHOD (PitchGet)       (THIS_ WORD *) PURE;
STDMETHOD (PitchSet)       (THIS_ WORD) PURE;
STDMETHOD (RealTimeGet)    (THIS_ DWORD *) PURE;
STDMETHOD (RealTimeSet)    (THIS_ DWORD) PURE;
STDMETHOD (SpeedGet)       (THIS_ DWORD *) PURE;
STDMETHOD (SpeedSet)       (THIS_ DWORD) PURE;
STDMETHOD (VolumeGet)      (THIS_ DWORD *) PURE;
STDMETHOD (VolumeSet)      (THIS_ DWORD) PURE;
};
typedef ITTSAttributesW FAR * PITTSATTRIBUTESW;

typedef struct _NotifySink
{
ITTSNotifySinkW iface;
} NotifySink;

static HRESULT STDMETHODCALLTYPE NotifySink_QueryInterface(ITTSNotifySinkW *This,REFIID riid,LPVOID *ppvObj)
{
*ppvObj=NULL;
if (IsEqualIID (riid, &IID_IUnknown) || IsEqualIID(riid, &IID_ITTSNotifySinkW))
{
*ppvObj=(LPVOID)This;
return S_OK;
}
return E_NOINTERFACE;
}

static ULONG STDMETHODCALLTYPE NotifySink_AddRef(ITTSNotifySinkW *This)
{
return 1;
}

static ULONG STDMETHODCALLTYPE NotifySink_Release(ITTSNotifySinkW *This)
{
return 1;
}

static HRESULT STDMETHODCALLTYPE NotifySink_AttribChanged(ITTSNotifySinkW *This,DWORD dwAttribute)
{
return S_OK;
}

static HRESULT STDMETHODCALLTYPE NotifySink_AudioStart(ITTSNotifySinkW *This,QWORD qTimeStamp)
{
return S_OK;
}

static HRESULT STDMETHODCALLTYPE NotifySink_AudioStop(ITTSNotifySinkW *This,QWORD qTimeStamp)
{
return S_OK;
}

static HRESULT STDMETHODCALLTYPE NotifySink_Visual(ITTSNotifySinkW *This,QWORD qTimeStamp,WCHAR cIPAPhoneme,WCHAR cEnginePhoneme,DWORD dwHints,LPVOID pTTSMouth)
{
return S_OK;
}

static ITTSNotifySinkWVtbl NotifySinkVtbl =
{
NotifySink_QueryInterface,
NotifySink_AddRef,
NotifySink_Release,
NotifySink_AttribChanged,
NotifySink_AudioStart,
NotifySink_AudioStop,
NotifySink_Visual
};

typedef struct _BufNotifySink
{
ITTSBufNotifySink iface;
TTSCALLBACK callback;
} BufNotifySink;

static HRESULT STDMETHODCALLTYPE BufNotifySink_QueryInterface(ITTSBufNotifySink *This,REFIID riid,LPVOID *ppvObj)
{
*ppvObj=NULL;
if (IsEqualIID(riid,&IID_IUnknown) || IsEqualIID(riid,&IID_ITTSBufNotifySink))
{
*ppvObj=(LPVOID)This;
return S_OK;
}
return E_NOINTERFACE;
}

static ULONG STDMETHODCALLTYPE BufNotifySink_AddRef(ITTSBufNotifySink *This)
{
return 1;
}

static ULONG STDMETHODCALLTYPE BufNotifySink_Release(ITTSBufNotifySink *This)
{
return 1;
}

static HRESULT STDMETHODCALLTYPE BufNotifySink_TextDataDone(ITTSBufNotifySink *This,QWORD qTimeStamp,DWORD dwFlags)
{
BufNotifySink *pInstance;
if ((pInstance=(BufNotifySink *)This)!=NULL)
{
if (pInstance->callback!=NULL)
pInstance->callback(TTS_EVENT_TEXTDONE);
}
return S_OK;
}

static HRESULT STDMETHODCALLTYPE BufNotifySink_TextDataStarted(ITTSBufNotifySink *This,QWORD qTimeStamp)
{
BufNotifySink *pInstance;
if ((pInstance=(BufNotifySink *)This)!=NULL)
{
if (pInstance->callback!=NULL)
pInstance->callback(TTS_EVENT_TEXTSTART);
}
return S_OK;
}

static HRESULT STDMETHODCALLTYPE BufNotifySink_BookMark(ITTSBufNotifySink *This,QWORD qTimeStamp,DWORD dwMarkNum)
{
return S_OK;
}

static HRESULT STDMETHODCALLTYPE BufNotifySink_WordPosition(ITTSBufNotifySink *This,QWORD qTimeStamp,DWORD dwByteOffset)
{
return S_OK;
}

static ITTSBufNotifySinkVtbl BufNotifySinkVtbl =
{
BufNotifySink_QueryInterface,
BufNotifySink_AddRef,
BufNotifySink_Release,
BufNotifySink_TextDataDone,
BufNotifySink_TextDataStarted,
BufNotifySink_BookMark,
BufNotifySink_WordPosition
};

typedef struct _TtsSynth
{
PITTSCENTRALW     pTts;
PITTSATTRIBUTESW  pAttr;
DWORD             dwReg;
DWORD             dwMinSpeed,dwMaxSpeed;
WORD              wMinPitch,wMaxPitch;
DWORD             dwMinVol,dwMaxVol;
NotifySink        notify;
BufNotifySink     bufnotify;
} TtsSynth;

//------------------------------------------- DLL Code ---------------------------------------------------

int __declspec(dllexport) Begin_Speech(DWORD bufSize)
{
if (SUCCEEDED(CoInitialize(NULL)))
{
initd=S_OK;
Memory_Handle=GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE,bufSize);
if (Memory_Handle)
{
Memory_LockPtr=(char *)GlobalLock(Memory_Handle);
if (Memory_LockPtr)
{
MemCopy_Handle=GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE,bufSize*sizeof(WCHAR));
if (MemCopy_Handle)
{
MemCopy_LockPtr=(char *)GlobalLock(MemCopy_Handle);
if (MemCopy_LockPtr)
{
memSize=bufSize;
return 1;
}
}
}
}
}
return 0;
}

int __declspec(dllexport) Count_Voices(void)
{
TTSENUM en;
TTSMODEINFOW info;
int counter=0;
if (initd==S_OK)
{
if ((en=tts_EnumCreate())!=NULL)
{
if (tts_EnumReset(en))
{
while (tts_EnumNext(en,&info))
{
counter++;
}
}
tts_EnumDelete(en);
}
}
return counter;
}

void __declspec(dllexport) Enum_VoiceData(void)
{
if (initd==S_OK)
vdPtr=Enum_VoiceInfo();
}

void __declspec(dllexport) Enum_VoiceNames(void)
{
if (initd==S_OK)
vnPtr=Enum_VoiceMode();
}

voiceData * __declspec(dllexport) Get_VoiceData(int index)
{
if (initd==S_OK)
{
if ((vdPtr) && (index>=0 && index<ARRAY_SIZE))
return &vdPtr[index];
}
return NULL;
}

voiceName * __declspec(dllexport) Get_VoiceName(int index)
{
if (initd==S_OK)
{
if ((vnPtr) && (index>=0 && index<ARRAY_SIZE))
return &vnPtr[index];
}
return NULL;
}

BOOL __declspec(dllexport) Speak_String(int uvt,char *voiceText,int cmb,int voiceNumber,int delkeep,int uvs,DWORD volume,int ups,DWORD pitch,int uss,DWORD speed)
{
TTSENUM en;
TTSMODEINFOW info;
HANDLE get_Handle=0;
int counter=0;
DWORD rtnval=0;
if (initd==S_OK)
{
if ((tts) && (delkeep==1))
{
tts_Delete(tts);
tts=NULL;
}
if (Memory_LockPtr)
{
if (cmb==1)
ZeroMemory(Memory_LockPtr,memSize);
else if (cmb==2)
ZeroMemory(MemCopy_LockPtr,memSize*sizeof(WCHAR));
else if (cmb==3)
{
ZeroMemory(Memory_LockPtr,memSize);
ZeroMemory(MemCopy_LockPtr,memSize*sizeof(WCHAR));
}
if (uvt)
{
if (OpenClipboard(NULL))
{
get_Handle=GetClipboardData(CF_TEXT);
if (get_Handle)
CopyMemory(Memory_LockPtr,get_Handle,strlen(get_Handle));
CloseClipboard();
}
}
else
CopyMemory(Memory_LockPtr,voiceText,strlen(voiceText));
if ((en=tts_EnumCreate())!=NULL)
{
if (tts_EnumReset(en))
{
while ((tts_EnumNext(en,&info)) && counter<voiceNumber)
{
counter++;
}
tts=tts_Create(&info,NULL);
if (tts!=NULL)
{
if (uvs)
tts_SetVolume(tts,volume);
if (ups)
tts_SetPitch(tts,pitch);
if (uss)
tts_SetSpeed(tts,speed);
rtnval=tts_TextOut(tts);
}
}
tts_EnumDelete(en);
}
}
}
return rtnval;
}

BOOL __declspec(dllexport) Pause_Speech(void)
{
if (initd==S_OK)
{
if (tts!=NULL)
return tts_Pause(tts);
}
return 0;
}

BOOL __declspec(dllexport) Stop_Speech(void)
{
if (initd==S_OK)
{
if (tts!=NULL)
return tts_Stop(tts);
}
return 0;
}

BOOL __declspec(dllexport) Resume_Speech(void)
{
if (initd==S_OK)
{
if (tts!=NULL)
return tts_Resume(tts);
}
return 0;
}

BOOL __declspec(dllexport) Tag_Speech(char * injstg)
{
WCHAR stg[2000];
if (initd==S_OK)
{
if (tts!=NULL)
{
MultiByteToWideChar(CP_ACP,0,injstg,-1,stg,strlen(injstg)*sizeof(WCHAR));
return tts_Inject(tts,stg);
}
}
return 0;
}

void __declspec(dllexport) Delete_Speech(void)
{
if (initd==S_OK)
{
if (tts!=NULL)
{
tts_Delete(tts);
tts=NULL;
}
}
}

DWORD __declspec(dllexport) Get_Volume(int voiceNumber)
{
TTSENUM en;
TTSMODEINFOW info;
int counter=0;
DWORD percentage=0;
if (initd==S_OK)
{
if (tts)
{
tts_Delete(tts);
tts=NULL;
}
if ((en=tts_EnumCreate())!=NULL)
{
if (tts_EnumReset(en))
{
while ((tts_EnumNext(en,&info)) && counter<voiceNumber)
{
counter++;
}
tts=tts_Create(&info,NULL);
if (tts!=NULL)
{
tts_GetVolume(tts,&percentage);
}
}
tts_EnumDelete(en);
}
}
return percentage;
}

DWORD __declspec(dllexport) Get_Pitch(int voiceNumber)
{
TTSENUM en;
TTSMODEINFOW info;
int counter=0;
DWORD percentage=0;
if (initd==S_OK)
{
if (tts)
{
tts_Delete(tts);
tts=NULL;
}
if ((en=tts_EnumCreate())!=NULL)
{
if (tts_EnumReset(en))
{
while ((tts_EnumNext(en,&info)) && counter<voiceNumber)
{
counter++;
}
tts=tts_Create(&info,NULL);
if (tts!=NULL)
{
tts_GetPitch(tts,&percentage);
}
}
tts_EnumDelete(en);
}
}
return percentage;
}

DWORD __declspec(dllexport) Get_Speed(int voiceNumber)
{
TTSENUM en;
TTSMODEINFOW info;
int counter=0;
DWORD percentage=0;
if (initd==S_OK)
{
if (tts)
{
tts_Delete(tts);
tts=NULL;
}
if ((en=tts_EnumCreate())!=NULL)
{
if (tts_EnumReset(en))
{
while ((tts_EnumNext(en,&info)) && counter<voiceNumber)
{
counter++;
}
tts=tts_Create(&info,NULL);
if (tts!=NULL)
{
tts_GetSpeed(tts,&percentage);
}
}
tts_EnumDelete(en);
}
}
return percentage;
}

void __declspec(dllexport) End_Speech(void)
{
if (initd==S_OK)
{
if (Memory_LockPtr)
GlobalUnlock(Memory_Handle);
if (Memory_Handle)
GlobalFree(Memory_Handle);
if (MemCopy_LockPtr)
GlobalUnlock(MemCopy_Handle);
if (MemCopy_Handle)
GlobalFree(MemCopy_Handle);
if (tts)
{
tts_Delete(tts);
tts=NULL;
}
CoUninitialize();
}
}

//-------------------------------------- Speech Functions -------------------------------------------------

TTSENUM tts_EnumCreate(void)
{
HRESULT hr;
ITTSEnumW *pEnum;
hr=CoCreateInstance(&CLSID_TTSEnumerator,NULL,CLSCTX_ALL,&IID_ITTSEnumW,(LPVOID *)&pEnum);
if (SUCCEEDED(hr))
return (TTSENUM) pEnum;
return NULL;
}

void tts_EnumDelete(TTSENUM ttsEnum)
{
ITTSEnumW *pEnum;
if ((pEnum=(ITTSEnumW *)ttsEnum)!=NULL)
pEnum->lpVtbl->Release(pEnum);
}

BOOL tts_EnumReset(TTSENUM ttsEnum)
{
ITTSEnumW *pEnum;
if ((pEnum=(ITTSEnumW *)ttsEnum)!=NULL)
return (SUCCEEDED(pEnum->lpVtbl->Reset(pEnum)));
return FALSE;
}

BOOL tts_EnumNext(TTSENUM ttsEnum,PTTSMODEINFOW pTtsModeInfo)
{
ITTSEnumW *pEnum;
UINT nCount=0;
if (((pEnum=(ITTSEnumW *)ttsEnum)!=NULL) && (pTtsModeInfo!=NULL))
{
if (SUCCEEDED(pEnum->lpVtbl->Next(pEnum,1,pTtsModeInfo,&nCount)) && (nCount==1))
return TRUE;
}
return FALSE;
}

TTS tts_Create(PTTSMODEINFOW pTtsModeInfo,TTSCALLBACK lpfnCallback)
{
TtsSynth *pSynth;
HRESULT hr;
ITTSEnumW *pEnum;
PIAUDIOMULTIMEDIADEVICE pIMMDevice;
IUnknown *pUnkAudio;
DWORD dwCur;
WORD wCur;
BOOL bOk;
if (pTtsModeInfo!=NULL)
{
pSynth=(TtsSynth *)malloc(sizeof(TtsSynth));
if (pSynth==NULL)
return NULL;
pSynth->pTts=NULL;
pSynth->notify.iface.lpVtbl=&NotifySinkVtbl;
pSynth->bufnotify.iface.lpVtbl=&BufNotifySinkVtbl;
pSynth->bufnotify.callback=lpfnCallback;
hr=CoCreateInstance(&CLSID_TTSEnumerator,NULL,CLSCTX_ALL,&IID_ITTSEnumW,(LPVOID *)&pEnum);
if (FAILED(hr))
{
free(pSynth);
return NULL;
}
hr=CoCreateInstance(&CLSID_MMAudioDest,NULL,CLSCTX_ALL,&IID_IAudioMultiMediaDevice,(LPVOID *)&pUnkAudio);
if (FAILED(hr))
{
free(pSynth);
pEnum->lpVtbl->Release(pEnum);
return FALSE;
}
if (FAILED(pUnkAudio->lpVtbl->QueryInterface(pUnkAudio,&IID_IAudioMultiMediaDevice,(void **)&pIMMDevice)))
{
free(pSynth);
pEnum->lpVtbl->Release(pEnum);
pUnkAudio->lpVtbl->Release(pUnkAudio);
return FALSE;
}
pIMMDevice->lpVtbl->DeviceNumSet(pIMMDevice,(UINT)-1);
pIMMDevice->lpVtbl->Release(pIMMDevice);
hr=pEnum->lpVtbl->Select(pEnum,pTtsModeInfo->gModeID,&pSynth->pTts,pUnkAudio);
pEnum->lpVtbl->Release(pEnum);
pUnkAudio->lpVtbl->Release(pUnkAudio);
if (SUCCEEDED(hr))
{
if (FAILED(pSynth->pTts->lpVtbl->Register(pSynth->pTts,(void *)&pSynth->notify,IID_ITTSNotifySinkW,&pSynth->dwReg)))
{
pSynth->pTts->lpVtbl->Release(pSynth->pTts);
free(pSynth);
return NULL;
}
if (FAILED(pSynth->pTts->lpVtbl->QueryInterface(pSynth->pTts,&IID_ITTSAttributesW,(PVOID*)&pSynth->pAttr)))
pSynth->pAttr=NULL;
else
{
bOk=(SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeGet(pSynth->pAttr,&dwCur)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeSet(pSynth->pAttr,0)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeGet(pSynth->pAttr,&pSynth->dwMinVol)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeSet(pSynth->pAttr,0xffffffff)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeGet(pSynth->pAttr,&pSynth->dwMaxVol)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeSet(pSynth->pAttr,dwCur)));
if (!bOk)
{
pSynth->dwMaxVol=0;
pSynth->dwMinVol=0;
}
bOk=(SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedGet(pSynth->pAttr,&dwCur)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedSet(pSynth->pAttr,0)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedGet(pSynth->pAttr,&pSynth->dwMinSpeed)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedSet(pSynth->pAttr,0xffffffff)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedGet(pSynth->pAttr,&pSynth->dwMaxSpeed)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedSet(pSynth->pAttr,dwCur)));
if (!bOk)
{
pSynth->dwMaxSpeed=0;
pSynth->dwMinSpeed=0;
}
bOk=(SUCCEEDED(pSynth->pAttr->lpVtbl->PitchGet(pSynth->pAttr,&wCur)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->PitchSet(pSynth->pAttr,0)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->PitchGet(pSynth->pAttr,&pSynth->wMinPitch)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->PitchSet(pSynth->pAttr,0xffff)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->PitchGet(pSynth->pAttr,&pSynth->wMaxPitch)));
bOk&=(SUCCEEDED(pSynth->pAttr->lpVtbl->PitchSet(pSynth->pAttr,wCur)));
if (!bOk)
{
pSynth->wMaxPitch=0;
pSynth->wMinPitch=0;
}
}
return (TTS)pSynth;
}
free(pSynth);
}
return NULL;
}

void tts_Delete(TTS Tts)
{
TtsSynth *pSynth;
if (((pSynth=(TtsSynth *)Tts)!=NULL) && (pSynth->pTts!=NULL))
{
pSynth->pTts->lpVtbl->AudioReset(pSynth->pTts);
pSynth->pTts->lpVtbl->UnRegister(pSynth->pTts,pSynth->dwReg);
if (pSynth->pAttr!=NULL)
pSynth->pAttr->lpVtbl->Release(pSynth->pAttr);
pSynth->pTts->lpVtbl->Release(pSynth->pTts);
free(pSynth);
}
}

BOOL tts_Stop(TTS Tts)
{
TtsSynth *pSynth;
if (((pSynth=(TtsSynth *)Tts)!=NULL) && (pSynth->pTts!=NULL))
return SUCCEEDED(pSynth->pTts->lpVtbl->AudioReset(pSynth->pTts));
return FALSE;
}

BOOL tts_Pause(TTS Tts)
{
TtsSynth *pSynth;
if (((pSynth=(TtsSynth *)Tts)!=NULL) && (pSynth->pTts!=NULL))
return SUCCEEDED(pSynth->pTts->lpVtbl->AudioPause(pSynth->pTts));
return FALSE;
}

BOOL tts_Resume(TTS Tts)
{
TtsSynth *pSynth;
if (((pSynth=(TtsSynth *)Tts)!=NULL) && (pSynth->pTts!=NULL))
return SUCCEEDED(pSynth->pTts->lpVtbl->AudioResume(pSynth->pTts));
return FALSE;
}

BOOL tts_Inject(TTS Tts,WCHAR * istg)
{
TtsSynth *pSynth;
if (((pSynth=(TtsSynth *)Tts)!=NULL) && (pSynth->pTts!=NULL))
return SUCCEEDED(pSynth->pTts->lpVtbl->Inject(pSynth->pTts,istg));
return FALSE;
}

BOOL tts_TextOut(TTS Tts)
{
TtsSynth *pSynth;
HRESULT hr;
SDATA text;
int nSize=0,rSize=0;
if (((pSynth=(TtsSynth *)Tts)!=NULL) && (pSynth->pTts!=NULL))
{
nSize=MultiByteToWideChar(CP_ACP,0,Memory_LockPtr,-1,NULL,0);
if (nSize)
{
text.dwSize=nSize*sizeof(WCHAR);
text.pData=MemCopy_LockPtr;
rSize=MultiByteToWideChar(CP_ACP,0,Memory_LockPtr,-1,(LPWSTR)text.pData,nSize);
if (rSize)
{
text.dwSize=(wcslen((LPWSTR)text.pData)+1)*sizeof(WCHAR);
if (text.dwSize<=memSize)
{
hr=pSynth->pTts->lpVtbl->TextData(pSynth->pTts,CHARSET_TEXT,0,text,(void *)&pSynth->bufnotify,IID_ITTSBufNotifySink);
return SUCCEEDED(hr);
}
}
}
}
return FALSE;
}

BOOL tts_SetVolume ( TTS Tts, UINT nVolPercent )
{
  TtsSynth *pSynth;
  DWORD dwNewVol;

  if ( ((pSynth = (TtsSynth *)Tts) != NULL) && (pSynth->pAttr != NULL) )
  {
    if ( (pSynth->dwMaxVol == 0) && (pSynth->dwMinVol == 0) )
    {
      return FALSE;
    }

    if ( nVolPercent == 0 )
    {
      dwNewVol = pSynth->dwMinVol;
    }
    else if ( nVolPercent >= 100 )
    {
      dwNewVol = pSynth->dwMaxVol;
    }
    else
    {
      dwNewVol = pSynth->dwMinVol + (DWORD)((((QWORD)pSynth->dwMaxVol - (QWORD)pSynth->dwMinVol) * (QWORD)nVolPercent) / (QWORD)100);
    }

    return SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeSet(pSynth->pAttr, dwNewVol));
  }

  return FALSE;
}

BOOL tts_GetVolume ( TTS Tts, UINT *pVolPercent )
{
  TtsSynth *pSynth;
  DWORD dwCurVol;

  if ( ((pSynth = (TtsSynth *)Tts) != NULL) && (pSynth->pAttr != NULL) && (pVolPercent != NULL) )
  {
    if ( (pSynth->dwMaxVol == 0) && (pSynth->dwMinVol == 0) )
    {
      return FALSE;
    }

    if ( SUCCEEDED(pSynth->pAttr->lpVtbl->VolumeGet(pSynth->pAttr, &dwCurVol)) )
    {
      *pVolPercent = (UINT)(((QWORD)dwCurVol * (QWORD)100) / ((QWORD)pSynth->dwMaxVol - (QWORD)pSynth->dwMinVol));
      return TRUE;
    }
  }

  return FALSE;
}

BOOL tts_SetSpeed ( TTS Tts, UINT nSpeedPercent )
{
  TtsSynth *pSynth;
  DWORD dwNewSpeed;

  if ( ((pSynth = (TtsSynth *)Tts) != NULL) && (pSynth->pAttr != NULL) )
  {
    if ( (pSynth->dwMaxSpeed == 0) && (pSynth->dwMinSpeed == 0) )
    {
      return FALSE;
    }

    if ( nSpeedPercent == 0 )
    {
      dwNewSpeed = pSynth->dwMinSpeed;
    }
    else if ( nSpeedPercent >= 100 )
    {
      dwNewSpeed = pSynth->dwMaxSpeed;
    }
    else
    {
      dwNewSpeed = pSynth->dwMinSpeed + (DWORD)((((QWORD)pSynth->dwMaxSpeed - (QWORD)pSynth->dwMinSpeed) * (QWORD)nSpeedPercent) / (QWORD)100);
    }

    return SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedSet(pSynth->pAttr, dwNewSpeed));
  }

  return FALSE;
}

BOOL tts_GetSpeed ( TTS Tts, UINT *pSpeedPercent )
{
  TtsSynth *pSynth;
  DWORD dwCurSpeed;

  if ( ((pSynth = (TtsSynth *)Tts) != NULL) && (pSynth->pAttr != NULL) && (pSpeedPercent != NULL) )
  {
    if ( (pSynth->dwMaxSpeed == 0) && (pSynth->dwMinSpeed == 0) )
    {
      return FALSE;
    }

    if ( SUCCEEDED(pSynth->pAttr->lpVtbl->SpeedGet(pSynth->pAttr, &dwCurSpeed)) )
    {
      *pSpeedPercent = (UINT)(((QWORD)dwCurSpeed * (QWORD)100) / ((QWORD)pSynth->dwMaxSpeed - (QWORD)pSynth->dwMinSpeed));
      return TRUE;
    }
  }

  return FALSE;
}

BOOL tts_SetPitch ( TTS Tts, UINT nPitchPercent )
{
  TtsSynth *pSynth;
  WORD wNewPitch;

  if ( ((pSynth = (TtsSynth *)Tts) != NULL) && (pSynth->pAttr != NULL) )
  {
    if ( (pSynth->wMaxPitch == 0) && (pSynth->wMinPitch == 0) )
    {
      return FALSE;
    }

    if ( nPitchPercent == 0 )
    {
      wNewPitch = pSynth->wMinPitch;
    }
    else if ( nPitchPercent >= 100 )
    {
      wNewPitch = pSynth->wMaxPitch;
    }
    else
    {
      wNewPitch = pSynth->wMinPitch + (WORD)((((DWORD)pSynth->wMaxPitch - (DWORD)pSynth->wMinPitch) * (DWORD)nPitchPercent) / (DWORD)100);
    }

    return SUCCEEDED(pSynth->pAttr->lpVtbl->PitchSet(pSynth->pAttr, wNewPitch));
  }

  return FALSE;
}

BOOL tts_GetPitch ( TTS Tts, UINT *pPitchPercent )
{
  TtsSynth *pSynth;
  WORD wCurPitch;

  if ( ((pSynth = (TtsSynth *)Tts) != NULL) && (pSynth->pAttr != NULL) && (pPitchPercent != NULL) )
  {
    if ( (pSynth->wMaxPitch == 0) && (pSynth->wMinPitch == 0) )
    {
      return FALSE;
    }

    if ( SUCCEEDED(pSynth->pAttr->lpVtbl->PitchGet(pSynth->pAttr, &wCurPitch)) )
    {
      *pPitchPercent = (UINT)(((DWORD)wCurPitch * (DWORD)100) / ((DWORD)pSynth->wMaxPitch - (QWORD)pSynth->wMinPitch));
      return TRUE;
    }
  }

  return FALSE;
}

char * UnicodeToAnsi(OLECHAR * szW)
{
static char szA[256];
szA[0]=0;
if (szW==NULL)
return "";
WideCharToMultiByte(CP_ACP,0,szW,-1,szA,256,NULL,NULL);
return szA;
}

voiceData * Enum_VoiceInfo(void)
{
TTSENUM en;
TTSMODEINFOW info;
int counter=0;
char * stgPtr;
if ((en=tts_EnumCreate())!=NULL)
{
if (tts_EnumReset(en))
{
while (tts_EnumNext(en,&info))
{
stgPtr=UnicodeToAnsi(info.szMfgName);
strcpy(vd[counter].maker,stgPtr);
stgPtr=UnicodeToAnsi(info.szProductName);
strcpy(vd[counter].product,stgPtr);
stgPtr=UnicodeToAnsi(info.szModeName);
strcpy(vd[counter].modeName,stgPtr);
stgPtr=UnicodeToAnsi(info.szSpeaker);
strcpy(vd[counter].speaker,stgPtr);
stgPtr=UnicodeToAnsi(info.szStyle);
strcpy(vd[counter].style,stgPtr);
stgPtr=UnicodeToAnsi(info.szDialect);
strcpy(vd[counter].dialect,stgPtr);
if (info.wGender==GENDER_FEMALE)
strcpy(vd[counter].gender,"Female");
else if (info.wGender==GENDER_MALE)
strcpy(vd[counter].gender,"Male");
else
strcpy(vd[counter].gender,"Gay!");
if (info.wAge==TTSAGE_BABY)
strcpy(vd[counter].age,"Baby  (1)");
else if (info.wAge==TTSAGE_TODDLER)
strcpy(vd[counter].age,"Toddler  (3)");
else if (info.wAge==TTSAGE_CHILD)
strcpy(vd[counter].age,"Child  (6)");
else if (info.wAge==TTSAGE_ADOLESCENT)
strcpy(vd[counter].age,"Adolescent  (14)");
else if (info.wAge==TTSAGE_ADULT)
strcpy(vd[counter].age,"Adult  (20-60)");
else if (info.wAge==TTSAGE_ELDERLY)
strcpy(vd[counter].age,"Elderly  (60+)");
else
strcpy(vd[counter].age,"Other");
vd[counter].features=info.dwFeatures;
vd[counter].features=info.dwInterfaces;
vd[counter].features=info.dwEngineFeatures;
counter++;
}
}
tts_EnumDelete(en);
}
return vd;
}

voiceName * Enum_VoiceMode(void)
{
TTSENUM en;
TTSMODEINFOW info;
int counter=0;
char * stgPtr;
if ((en=tts_EnumCreate())!=NULL)
{
if (tts_EnumReset(en))
{
while (tts_EnumNext(en,&info))
{
stgPtr=UnicodeToAnsi(info.szModeName);
strcpy(vn[counter].modeName,stgPtr);
counter++;
}
}
tts_EnumDelete(en);
}
return vn;
}

//------------------------------------------ Default DLL Code ---------------------------------------------

BOOL WINAPI __declspec(dllexport) LibMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
{
switch (fdwReason)
{
case DLL_PROCESS_ATTACH:
// The DLL is being loaded for the first time by a given process.
// Perform per-process initialization here.
// If the initialization is successful, return TRUE. Otherwise. return FALSE.
hInstance=hDLLInst;
break;
case DLL_PROCESS_DETACH:
// The DLL is being unloaded by a given process.
// Do any per-process clean up here, such as undoing what was done in DLL_PROCESS_ATTACH.
// The return value is ignored.
break;
case DLL_THREAD_ATTACH:
// A thread is being created in a process that has already loaded this DLL.
// Perform any per-thread initialization here. The return value is ignored.
break;
case DLL_THREAD_DETACH:
// A thread is exiting cleanly in a process that has already loaded this DLL.
// Perform any per-thread clean up here. The return value is ignored.
break;
}
return TRUE;
}
